package rione.agent.at;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.Collection;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.misc.gui.ScreenTransform;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import rione.agent.DebugAgent;
import rione.model.AdvancedWorldModel;
import rione.model.AgentConstants;
import rione.model.route.pov.CostFunction;
import rione.viewer.component.AdvancedViewComponent;
import rione.viewer.component.TransformedGraphics;
import rione.viewer.component.extension.BorderExtension;
import rione.viewer.component.extension.ExtensionMap;
import rione.viewer.component.extension.TextExtension;

/**
 * デバッグ時は実行の構成の起動オプションからこちらを起動する
 * @author utisam
 *
 */
public class DebugAmbulanceTeamAgent extends AmbulanceTeamAgent implements DebugAgent {

	/** 最後に呼び出されたメソッドを表す文字列 */
	private String lastCommand = null;

	@Override
	protected void initialize() {
		super.initialize();
		//ビューアに登録
		AdvancedViewComponent.addTeamAgent(this);
	}

	@Override
	public AdvancedWorldModel getWorld() {
		return world;
	}

	/*
	 * 自由に設定できるEntityIDと装飾の対応
	 * (非 Javadoc)
	 * @see rione.viewer.component.DebugAgent#customExtension()
	 */
	@Override
	public ExtensionMap customExtension() {
		ExtensionMap exMap = new ExtensionMap();

		world.getBuriedHumans().extension(exMap);
		for(Building building : PrioritizeBuilding){
			exMap.put(building, new BorderExtension(exMap.get(building),Color.green, 3.0f));
		}
		
		for (StandardEntity se : world.getEntitiesOfType(AgentConstants.HUMANOIDS)) {
			Human human = (Human)se;
			//クリックしたAgentを表示
			if(se.getID() == me().getID() && location() instanceof Road){
				exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
						"ID: " + me().getID(), Color.YELLOW));
				if (human.isHPDefined() && human.isDamageDefined()){ 
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"HP: " + human.getHP(), Color.YELLOW));
					if(human.getDamage() > 0)
						exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
								"Damage: " + human.getDamage(), Color.YELLOW));
				}
			}
			//埋まっているHumanのHPとDamageとBuriednessを表示
			if(isVisible(se)){
				if(human.isHPDefined() && human.isDamageDefined() && human.isBuriednessDefined()
						&& human.getBuriedness() > 0){
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"HP: " + human.getHP(), Color.YELLOW));
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"Damage: " + human.getDamage(), Color.YELLOW));
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"Buriedness: " + human.getBuriedness(), Color.YELLOW));
				}
			}
		}
		/*for(StandardEntity road : unlookupedRoads){
			exMap.put(road, new BorderExtension(exMap.get(road),Color.blue, 3.0f));
		}*/
		return exMap;
	}

	/*
	 * 自由に設定できる描画処理
	 * (非 Javadoc)
	 * @see rione.agent.DebugAgent#customRender(java.awt.Graphics2D, rescuecore2.misc.gui.ScreenTransform, int, int)
	 */
	@Override
	public void customRender(Graphics2D g, ScreenTransform t, int width, int height) {
		/*TransformedGraphics gt = new TransformedGraphics(g, t);
		router.getPassableDic().customRender(gt);*/
		g.setFont(new Font(g.getFont().getFontName(),Font.BOLD,20));
		TransformedGraphics gt = new TransformedGraphics(g, t);
		gt.setThickness(4);
		router.getPassableDic().customRender(gt);
	}

	//_/_/_/_/_/_/_/_/_/_/_/_/_/
	//_/ 以下CommandCall用 /_/
	//_/_/_/_/_/_/_/_/_/_/_/_/

	@Override
	public String getCommandsCall() {
		if (lastCommand == null) {
			return "";
		}
		return lastCommand;
	}

	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
		long thinkStart = System.currentTimeMillis();
		super.think(time, changed, heard);
		lastCommand += ": " + (System.currentTimeMillis() - thinkStart) + "[ms]";
	}

	@Override
	protected void act() throws ActionCommandException {
		lastCommand = "act";
		super.act();
	}

	// 以下自動生成

	@Override
	protected void rest() throws ActionCommandException {
		lastCommand += "->rest";
		super.rest();
	}
	@Override
	protected void moveFront(StandardEntity destination, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destination, costFunc);
	}
	@Override
	protected void move(List<EntityID> path, int x, int y) throws ActionCommandException {
		lastCommand += "->move";
		super.move(path, x, y);
	}
	@Override
	protected void moveFront(Collection<? extends StandardEntity> destinations) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destinations);
	}
	@Override
	protected void moveFront(Collection<? extends StandardEntity> destinations, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destinations, costFunc);
	}
	@Override
	protected void moveFront(List<EntityID> path) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(path);
	}
	@Override
	protected void moveFront(List<EntityID> path, int x, int y) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(path, x, y);
	}
	@Override
	protected void move(StandardEntity destination, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->move";
		super.move(destination, costFunc);
	}
	@Override
	protected void move(StandardEntity destination) throws ActionCommandException {
		lastCommand += "->move";
		super.move(destination);
	}
	@Override
	protected void moveReachable(Collection<? extends StandardEntity> destinations, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(destinations, front);
	}
	@Override
	protected void moveReachable(Collection<? extends StandardEntity> destinations, CostFunction costFunc, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(destinations, costFunc, front);
	}
	@Override
	protected void moveReachable(List<EntityID> path, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(path, front);
	}
	@Override
	protected void moveReachable(List<EntityID> path, int x, int y, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(path, x, y, front);
	}
	@Override
	public void MoveToError() throws ActionCommandException {
		lastCommand += "->MoveToError";
		super.MoveToError();
	}
	@Override
	protected void goOut(Building bld) throws ActionCommandException {
		lastCommand += "->goOut";
		super.goOut(bld);
	}
	@Override
	protected void lookupSearchBuildings() throws ActionCommandException {
		lastCommand += "->lookupSearchBuildings";
		super.lookupSearchBuildings();
	}
	@Override
	protected void enterSearchBuildings() throws ActionCommandException {
		lastCommand += "->enterSearchBuildings";
		super.enterSearchBuildings();
	}
	@Override
	protected void lookupSearchRoads() throws ActionCommandException {
		lastCommand += "->lookupSearchRoads";
		super.lookupSearchRoads();
	}
	@Override
	protected void randomWalk() throws ActionCommandException {
		lastCommand += "->randomWalk";
		super.randomWalk();
	}
	@Override
	protected void moveToRefuge() throws ActionCommandException {
		lastCommand += "->moveToRefuge";
		super.moveToRefuge();
	}
	@Override
	protected void aggregatorStay() throws ActionCommandException {
		lastCommand += "->aggregatorStay";
		super.aggregatorStay();
	}
	@Override
	protected void messengerLoop(int interval) throws ActionCommandException {
		lastCommand += "->messengerLoop";
		super.messengerLoop(interval);
	}
	@Override
	protected void careSelf() throws ActionCommandException {
		lastCommand += "->careSelf";
		super.careSelf();
	}
	@Override
	protected void prioritize() throws ActionCommandException {
		lastCommand += "->prioritize";
		super.prioritize();
	}
	@Override
	protected void rescue(EntityID target) throws ActionCommandException {
		lastCommand += "->rescue";
		super.rescue(target);
	}
	@Override
	protected void load(EntityID target) throws ActionCommandException {
		lastCommand += "->load";
		super.load(target);
	}
	@Override
	protected void unload() throws ActionCommandException {
		lastCommand += "->unload";
		super.unload();
	}
	@Override
	protected void loadOrRescueAround() throws ActionCommandException {
		lastCommand += "->loadOrRescueAround";
		super.loadOrRescueAround();
	}
	@Override
	protected void moveToTarget() throws ActionCommandException {
		lastCommand += "->moveToTarget";
		super.moveToTarget();
	}
	@Override
	protected void carry() throws ActionCommandException {
		lastCommand += "->carry";
		super.carry();
	}
	@Override
	protected void matchingATandVictim() throws ActionCommandException {
		lastCommand += "->matchingATandVictim";
		super.matchingATandVictim();
	}

}
