package rione.agent.pf;

import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.util.*;

import com.infomatiq.jsi.Point;

import rione.model.route.pov.POVRouter;
import rescuecore2.misc.Pair;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.messages.StandardMessageURN;
import rescuecore2.worldmodel.EntityID;
import rione.agent.PlatoonAgent;
import rione.geom.ExpandApexes;
import rione.model.Complex;
import rione.model.route.pov.CostFunction;
import rione.model.route.pov.POVPath;
import rione.model.route.pov.graph.AreaNode;
import rione.model.route.pov.graph.EdgeNode;
import rione.model.route.pov.graph.PassableDictionary;
import rione.model.route.pov.graph.PassableDictionary.PassableLevel;
import rione.model.route.pov.graph.PointNode;
import rione.util.SetPair;

public abstract class AbstractPoliceForceAgent extends
PlatoonAgent<PoliceForce> {

	protected Set<Blockade> clearedBlockade = new HashSet<Blockade>();

	/**
	 * 座標を渡したClear
	 * 
	 * @param target
	 *            Clearする対象のID
	 * @throws ActionCommandException
	 */
	protected void clear(int x,int y) throws ActionCommandException {
		sendClear(time, x, y);
		throw new ActionCommandException(StandardMessageURN.AK_CLEAR);
	}

	/**
	 * Pairを渡したClear
	 * 
	 * @param block
	 *            Clearする対象
	 * @throws ActionCommandException
	 */
	protected void clear(Pair<Integer,Integer> position) throws ActionCommandException {
		clear(position.first(),position.second());
	}

	/**
	 * ある閉塞が指定したPoliceForceAgentが 啓開できる距離にあるかどうかを判断する
	 * 
	 * @param pf
	 *            指定したPoliceForceAgent
	 * @param block
	 *            判断する閉塞
	 * @return 閉塞が啓開できる距離にあるとき true
	 */
	protected boolean isClearable(PoliceForce pf, Blockade block) {
		Pair<Integer, Integer> position = pf.getLocation(world);
		final int r = 1000;
		java.awt.geom.Area range = new java.awt.geom.Area(new Ellipse2D.Double(
				position.first() - r, position.second() - r,
				r * 2, r * 2));
		if(block==null || block.getPosition()==null)return false;
		if(block.getShape() == null)return false;
		range.intersect(new java.awt.geom.Area(block.getShape()));
		return !range.isEmpty();
	}

	protected void expandClear(int x, int y) throws ActionCommandException{
		
		int clearRate = world.getConfig().repairDistance + 2000;
		float zavie = (float) Math.atan2(y - me().getY(), x - me().getX());
		Pair<Integer, Integer> clearLine = new Pair<Integer, Integer>((int) (me().getX() + clearRate * Math.cos(zavie)),  (int) (me().getY() + clearRate * Math.sin(zavie)));
		clear(clearLine.first(), clearLine.second());
	}

	/**
	 * 近くの瓦礫の除去を行う
	 * @param block
	 * @throws ActionCommandException
	 */
	protected void expandClear(Blockade block) throws ActionCommandException{
		if (block.getShape() == null) {
			return;
		}
		expandClear(block.getX(), block.getY());
		//sendClear(time, block.getID());
	}

	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.POLICE_FORCE);
	}

	/*
	 * moveを上書きしてsmartMoveで横取りする
	 * (非 Javadoc)
	 * @see rione.agent.HumanoidAgent#move(rescuecore2.standard.entities.StandardEntity, rione.model.route.pov.CostFunction)
	 */
	@Override
	protected void move(StandardEntity destination, CostFunction costFunc) throws ActionCommandException {
		POVPath path = null;
		if (destination instanceof Human) {
			Human human = (Human) destination;
			Area position = null;
			try {
				position = (Area) human.getPosition(world);
			}
			catch (ClassCastException e) {
				position = (Area) ((Human) (human.getPosition(world))).getPosition(world);
			}
			if (position == null) return;
			path = router.search().getAStarPath(
					me(),
					position,
					costFunc);
		}else if (destination instanceof Blockade) {
			Blockade block = (Blockade)destination;
			EntityID position = block.getPosition();
			if (position == null) return;
			path = router.search().getAStarPath(
					me(),
					(Area) world.getEntity(position),
					costFunc);
		}else {
			path = router.search().getAStarPath(
					me(),
					(Area) destination,
					costFunc);
		}
		smartMove(path);
	}
	/*
	 * moveを上書きしてsmartMoveで横取りする
	 * (非 Javadoc)
	 * @see rione.agent.HumanoidAgent#move(java.util.Collection, rione.model.route.pov.CostFunction)
	 */ 
	@Override
	protected void move(Collection<? extends StandardEntity> destinations, CostFunction costFunc) throws ActionCommandException {
		POVPath path = router.search().getDijkstraPath(
				location(),
				new HashSet<StandardEntity>(destinations),
				costFunc);
		smartMove(path);
	}

	protected void smartMove(POVPath path) throws ActionCommandException {
		if (path == null) {
			rest();
		}
		else {
			clearPath(path);
			move(path.getRoute());
		}
	}

	/**
	 * pathと交差したBlockadeのみを消す、座標指定型のClear
	 * @param path
	 * @throws rione.agent.Agent.ActionCommandException
	 */
	protected void clearPath(final POVPath path) throws rione.agent.Agent.ActionCommandException {
		//TODO 座標指定のClearの作成
		// ディープコピーを作成
		List<PointNode> points = new ArrayList<PointNode>(path.getPoints());
		Pair<Integer, Integer> loc = me().getLocation(world);
		Pair <Integer,Integer> clearLine = new Pair<Integer, Integer>(-1,-1);
		Area area = location();

		//自分と同じ場所にBlockadeがあるなら
		if (area.isBlockadesDefined()) {
			for (EntityID id : area.getBlockades()) {
				Blockade block = (Blockade) world.getEntity(id);
				if(!isClearable(me(), block))continue;


				//除去を行う座標を取得
				clearLine = hasCross(block,
						new PointNode(false, loc.first(), loc.second()),points.get(0));
				if((clearLine.first() == -1 && clearLine.second() == -1))continue;

				int x = clearLine.first();
				int y = clearLine.second();

				PointNode node0 = points.get(0);
				PointNode node1 = points.get(1);
				

				if(router.getPassableDic().getPassableLevel(node0, node1)==PassableLevel.UNPASSABLE || 
						router.getPassableDic().getPassableLevel(node0,node1)==PassableLevel.LOGICAL_PASSABLE){
					x = loc.first()+(node1.getX()-node0.getX());y = loc.second()+(node1.getY()-node0.getY());
				}
				else {
					int diffX = node1.getX() - node0.getX();
					int diffY = node1.getY() - node0.getY();
					x = loc.first() + diffX; y = loc.second() + diffY;	
				}


				StandardEntity se = world.getEntity(block.getPosition());
				Area blockArea = (Area)se;
				if(getCommandHistory(time - 1) == null || getCommandHistory(time - 2) == null)continue;
				if(getCommandHistory(time - 1).equals(StandardMessageURN.AK_CLEAR)
						&& getCommandHistory(time - 2).equals(StandardMessageURN.AK_CLEAR)){
					move(blockArea,x,y);
				}
				expandClear(x,y);
				//sendClear(time, id);
			}

		}

		PointNode previous = points.get(0);
		for (int i = 1; i < points.size(); i++) {
			PointNode current = points.get(i);
			area = getArea(current, previous);
			if (area.isBlockadesDefined()) {
				for (EntityID id : area.getBlockades()) {
					Blockade block = (Blockade) world.getEntity(id);
					if(!isClearable(me(), block))continue;

					//除去を行う座標を取得
					clearLine = hasCross(block, current, previous);
					if((clearLine.first() == -1 && clearLine.second() == -1))continue;

					int x = clearLine.first();
					int y = clearLine.second();
					if(i==points.size()) continue;
			


						if((router.getPassableDic().getPassableLevel(previous, current)==PassableLevel.UNPASSABLE ||
								router.getPassableDic().getPassableLevel(previous, current)==PassableLevel.LOGICAL_PASSABLE)){
							x=loc.first()+(current.getX()-previous.getX());y=loc.second()+(current.getY()-previous.getY());
						}
						else {
							int diffX = current.getX() - previous.getX();
							int diffY = current.getY() - previous.getY();
							x = loc.first() + diffX; y = loc.second() + diffY;
						}
		
			
					StandardEntity se = world.getEntity(block.getPosition());
					Area blockArea = (Area)se;
					if(getCommandHistory(time - 1) == null || getCommandHistory(time - 2) == null)continue;
					if(getCommandHistory(time - 1).equals(StandardMessageURN.AK_CLEAR)
							&& getCommandHistory(time - 2).equals(StandardMessageURN.AK_CLEAR)){
						move(blockArea,x,y);
					}
					expandClear(x, y);
					//sendClear(time, id);
				}
			}
			previous = current;
		}
	}

	/**自分の位置とBlockadeのpathが交差しているかどうか*/
	private Pair<Integer,Integer> hasCross(Blockade block, PointNode current, PointNode previous) {
		//TODO 座標指定の瓦礫の判定
		final Shape shape = ExpandApexes.expandApexes(block, 0);
		if (shape == null) return  new Pair<Integer, Integer>(-1,-1);

		//対象の瓦礫を拡大した場合に、その瓦礫に含まれるかどうか
		if (shape.contains(current.getX(), current.getY())
				|| shape.contains(previous.getX(), previous.getY())
				|| shape.contains(me().getX(), me().getY())) {
			int shapeX = block.getX() - me().getX();
			int shapeY = block.getY() - me().getY();
			//System.out.println(time+" GOTO shape ID:"+me().getID());
			return new Pair<Integer, Integer>(block.getX() + shapeX*2,block.getY() + shapeY*2);
		}
		final Line2D pathLine = new Line2D.Double(
				current.getX(), current.getY(),
				previous.getX(), previous.getY());
		PathIterator it = shape.getPathIterator(null);
		double[] c = new double[6];
		double x = 0.0, y = 0.0;
		while (!it.isDone()) {
			switch (it.currentSegment(c)) {
			case PathIterator.SEG_MOVETO:
				x = c[0];
				y = c[1];
				break;
			case PathIterator.SEG_LINETO:
				//交差しているかどうか
				if (pathLine.intersectsLine(x, y, c[0], c[1])) {
					//交差している点のうちのどれかが、Buildingと同じ(除去の目的地が建物)ならそちらに向かって除去をする
					/*for (EntityID id : getVisibleEntities()) {
						StandardEntity se = world.getEntity(id);
						if(se instanceof Building){
							Building building = (Building)se;
							if((building.getX() - x < 10 && building.getY() - y < 10)
									|| (building.getX() - c[0] < 10 && building.getY() - c[1] <10)){
								//System.out.println(time + " GOTO Building ID:" + me().getID());
								return new Pair<Integer, Integer>(building.getX(),building.getY());
							}
						}
					}*/
					//交差した点に向かって除去を行う
					Complex meCom = new Complex((double)current.getX(), (double)current.getY());
					Complex bloCom = new Complex((double)previous.getX(), (double)previous.getY());
					Complex xyCom = new Complex(x, y);
					Complex ccCom = new Complex(c[0], c[1]);
					Complex target = Complex.intersection(meCom, bloCom, xyCom, ccCom);
					int getx = (int)target.real();
					int gety = (int)target.imag();
					if(getx == 0 && gety == 0){
						return new Pair<Integer, Integer>(-1,-1);
					}
					//System.out.println(time+" GOTO LINE ID:"+me().getID());
					return new Pair<Integer, Integer>(getx,gety);
				}
				x = c[0];
				y = c[1];
				break;
			case PathIterator.SEG_QUADTO:
			case PathIterator.SEG_CUBICTO:
			case PathIterator.SEG_CLOSE:
				break;
			}
			it.next();
		}
		return new Pair<Integer, Integer>(-1,-1);

	}


	private Area getArea(PointNode p1, PointNode p2) {
		AreaNode areaNode = null;
		if (p1 instanceof AreaNode) areaNode = (AreaNode) p1;
		if (p2 instanceof AreaNode) areaNode = (AreaNode) p2;
		if (areaNode == null) return null;
		return areaNode.getBelong();
	}
}
