package rione.agent.pf;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.misc.Pair;
import rescuecore2.misc.gui.ScreenTransform;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import rione.agent.DebugAgent;
import rione.model.AdvancedWorldModel;
import rione.model.AgentConstants;
import rione.model.route.pov.CostFunction;
import rione.model.route.pov.POVPath;
import rione.viewer.component.AdvancedViewComponent;
import rione.viewer.component.TransformedGraphics;
import rione.viewer.component.extension.BorderExtension;
import rione.viewer.component.extension.ExtensionMap;
import rione.viewer.component.extension.TextExtension;

/**
 * デバッグ時は実行の構成の起動オプションからこちらを起動する
 * @author utisam
 *
 */
public class DebugPoliceForceAgent extends PoliceForceAgent implements DebugAgent {

	/** 最後に呼び出されたメソッドを表す文字列 */
	private String lastCommand = null;
	
	@Override
	protected void initialize() {
		super.initialize();
		//ビューアに登録
		AdvancedViewComponent.addTeamAgent(this);
	}

	@Override
	public AdvancedWorldModel getWorld() {
		return world;
	}

	//private DynamicConstant enableReachableExtension = DynamicConstant.create("'ReachableExtension(pf)', false");

	/*
	 * 自由に設定できるEntityIDと装飾の対応
	 * (非 Javadoc)
	 * @see rione.viewer.component.DebugAgent#customExtension()
	 */
	@Override
	public ExtensionMap customExtension() {
		ExtensionMap exMap = new ExtensionMap();
		world.getBurningBuildings().extensions(exMap);
		world.getBuriedHumans().extension(exMap);
		

		for (StandardEntity se : world.getEntitiesOfType(AgentConstants.HUMANOIDS)) {
			Human human = (Human)se;
			//クリックしたAgentを表示
			if(se.getID() == me().getID()){
				exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
						"ID: " + me().getID(), Color.YELLOW));
				if (human.isHPDefined() && human.isDamageDefined()){ 
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"HP: " + human.getHP(), Color.YELLOW));
					if(human.getDamage() > 0)
					exMap.put(se.getID(), new TextExtension(exMap.get(se.getID()),
							"Damage: " + human.getDamage(), Color.YELLOW));
				}
			}
		}
		/*for(StandardEntity road : unlookupedRoads){
			exMap.put(road, new BorderExtension(exMap.get(road),Color.blue, 3.0f));
		}*/
		for(Building building : world.getBurningBuildings()){
			for(StandardEntity neighBurn : world.getNeighbours(building,10)){
				if(neighBurn instanceof Road){
					for(StandardEntity neighRoad : world.getNeighbours(neighBurn, 10)){
						Area area = (Area)neighRoad;
						if(neighRoad instanceof Road && !router.isSureReachable(area)){
							exMap.put(neighRoad, new BorderExtension(exMap.get(neighRoad),
									Color.RED, 3.0f));
						}
					}
				}
			}
		}
		for(Building building : PrioritizeBuilding){
			exMap.put(building, new BorderExtension(exMap.get(building),Color.green, 3.0f));
		}
		return exMap;
	}

	@Override
	public void customRender(Graphics2D g, ScreenTransform t, int width, int height) {
		g.setFont(new Font(g.getFont().getFontName(),Font.BOLD,20));
		TransformedGraphics gt = new TransformedGraphics(g, t);
		gt.setThickness(4);
	}

	//_/_/_/_/_/_/_/_/_/_/_/_/_/
	//_/ 以下CommandCall用 /_/
	//_/_/_/_/_/_/_/_/_/_/_/_/

	@Override
	public String getCommandsCall() {
		if (lastCommand == null) {
			return "";
		}
		return lastCommand;
	}

	long thinkStart = 0;
	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
		thinkStart = System.currentTimeMillis();
		super.think(time, changed, heard);
		lastCommand += ": " + (System.currentTimeMillis() - thinkStart) + "[ms]";
	}

	@Override
	protected void act() throws ActionCommandException {
		lastCommand = "act";
		super.act();
	}

	// 以下自動生成

	@Override
	protected void rest() throws ActionCommandException {
		lastCommand += "->rest";
		super.rest();
	}
	@Override
	protected void moveFront(StandardEntity destination, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destination, costFunc);
	}
	@Override
	protected void move(List<EntityID> path, int x, int y) throws ActionCommandException {
		lastCommand += "->move";
		super.move(path, x, y);
	}
	@Override
	protected void moveFront(Collection<? extends StandardEntity> destinations) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destinations);
	}
	@Override
	protected void moveFront(Collection<? extends StandardEntity> destinations, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(destinations, costFunc);
	}
	@Override
	protected void moveFront(List<EntityID> path) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(path);
	}
	@Override
	protected void moveFront(List<EntityID> path, int x, int y) throws ActionCommandException {
		lastCommand += "->moveFront";
		super.moveFront(path, x, y);
	}
	@Override
	protected void move(StandardEntity destination, CostFunction costFunc) throws ActionCommandException {
		lastCommand += "->move";
		super.move(destination, costFunc);
	}
	@Override
	protected void move(StandardEntity destination) throws ActionCommandException {
		lastCommand += "->move";
		super.move(destination);
	}
	@Override
	protected void moveReachable(Collection<? extends StandardEntity> destinations, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(destinations, front);
	}
	@Override
	protected void moveReachable(Collection<? extends StandardEntity> destinations, CostFunction costFunc, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(destinations, costFunc, front);
	}
	@Override
	protected void moveReachable(List<EntityID> path, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(path, front);
	}
	@Override
	protected void moveReachable(List<EntityID> path, int x, int y, boolean front) throws ActionCommandException {
		lastCommand += "->moveReachable";
		super.moveReachable(path, x, y, front);
	}
	@Override
	protected void goOut(Building bld) throws ActionCommandException {
		lastCommand += "->goOut";
		super.goOut(bld);
	}
	@Override
	protected void lookupSearchBuildings() throws ActionCommandException {
		lastCommand += "->lookupSearchBuildings";
		super.lookupSearchBuildings();
	}
	@Override
	protected void enterSearchBuildings() throws ActionCommandException {
		lastCommand += "->enterSearchBuildings";
		super.enterSearchBuildings();
	}
	@Override
	protected void randomWalk() throws ActionCommandException {
		lastCommand += "->randomWalk";
		super.randomWalk();
	}
	@Override
	protected void moveToRefuge() throws ActionCommandException {
		lastCommand += "->moveToRefuge";
		super.moveToRefuge();
	}
	@Override
	protected void aggregatorStay() throws ActionCommandException {
		lastCommand += "->aggregatorStay";
		super.aggregatorStay();
	}
	@Override
	protected void messengerLoop(int interval) throws ActionCommandException {
		lastCommand += "->messengerLoop";
		super.messengerLoop(interval);
	}
	@Override
	protected void expandClear(Blockade block) throws ActionCommandException {
		lastCommand += "->expandClear";
		super.expandClear(block);
	}
	@Override
	protected void clear(Pair<Integer,Integer> position) throws ActionCommandException {
		lastCommand += "->clear";
		super.clear(position);
	}
	@Override
	protected void smartMove(POVPath path) throws ActionCommandException {
		lastCommand += "->smartMove";
		super.smartMove(path);
	}
	@Override
	protected void clearPartedArea() throws ActionCommandException {
		lastCommand += "->clearPartedArea";
		super.clearPartedArea();
	}
	@Override
	protected void searchFires() throws ActionCommandException {
		lastCommand += "->searchFires";
		super.searchFires();
	}
	@Override
	protected void moveToLocateInBlockade() throws ActionCommandException {
		lastCommand += "->moveToLocateInBlockade";
		super.moveToLocateInBlockade();
	}
	@Override
	protected void clearNearLocateInBlockade() throws ActionCommandException {
		lastCommand += "->clearNearLocateInBlocade";
		super.clearNearLocateInBlockade();
	}
	@Override
	protected void eraseUnreachable() throws ActionCommandException {
		lastCommand += "->eraseUnreachable";
		super.eraseUnreachable();
	}
	@Override
	protected void clearBuriedHuman() throws ActionCommandException {
		lastCommand += "->clearBuriedHuman";
		super.clearBuriedHuman();
	}
	@Override
	protected void clearAround() throws ActionCommandException {
		lastCommand += "->clearAround";
		super.clearAround();
	}
	@Override
	protected void clearCriticalArea() throws ActionCommandException {
		lastCommand += "->clearCriticalArea";
		super.clearCriticalArea();
	}
	@Override
	protected void helpAgents() throws ActionCommandException {
		lastCommand += "->helpAgents";
		super.helpAgents();
	}
	@Override
	protected void prioritizeClear() throws ActionCommandException {
		lastCommand += "->prioritizeClear";
		super.prioritizeClear();
	}
		@Override
	protected void lookupSearchRoads() throws ActionCommandException {
		lastCommand += "->lookupSearchRoads";
		super.lookupSearchRoads();
	}
		@Override
	protected void clearAroundBurningBuilding() throws ActionCommandException {
		lastCommand += "->clearAroundBurningBuilding";
		super.clearAroundBurningBuilding();
	}
		@Override
	protected void clearNearRefuge() throws ActionCommandException {
		lastCommand += "->clearNearRefuge";
		super.clearNearRefuge();
	}
	@Override
	protected void clearNearHydrant() throws ActionCommandException {
		lastCommand += "->clearNearHydrant";
		super.clearNearHydrant();
	}
	@Override
	protected void clearHydrant() throws ActionCommandException {
		lastCommand += "->clearHydrant";
		super.clearHydrant();
	}
	@Override
	protected void cantMove() throws ActionCommandException {
		lastCommand += "->cantMove";
		super.cantMove();
	}
	/*@Override
	protected void careSelf() throws ActionCommandException {
		lastCommand += "->careSelf";
		super.careSelf();
	}*/

}
