package rione.communication;

import java.util.ArrayList;

import rescuecore2.standard.entities.StandardEntity;
import rione.communication.channel.RandomChannelManager;
import rione.communication.channel.ChannelManager;
import rione.model.AdvancedWorldModel;

public class CommunicationManagerPartsBuilder {

	private final AdvancedWorldModel world;
	private final StandardEntity me;
	
	private final ArrayList<Port> radioEventListener = new ArrayList<Port>();
	private Port[] radioEventListenerResult;
	private final ArrayList<Port> voiceEventListener = new ArrayList<Port>();
	private Port[] voiceEventListenerResult;
	private final CommunicationUtil comUtil;
	private final ArrayList<CivilianVoiceListener> civilianVoiceListener = new ArrayList<CivilianVoiceListener>();
	private CivilianVoiceListener[] civilianListenerResult;
	private ChannelManager channelManager;

	public CommunicationManagerPartsBuilder(final StandardEntity me, final AdvancedWorldModel world) {
		this.world = world;
		this.me = me;
		this.comUtil = new CommunicationUtil(world);
	}
	
	public void addRadioEventListener(Port l) {
		radioEventListener.add(l);
	}
	public void addVoiceEventListener(Port l) {
		voiceEventListener.add(l);
	}
	public void addCivilianVoiceListener(CivilianVoiceListener l) {
		civilianVoiceListener.add(l);
	}
	
	void build() {
		// CommunicationEventListener郡の生成
		// Centerを含めた全てのエージェントで同じ配列が生成されなくてはいけない
		// 各Listenerを個別に追加してもおｋ
		radioEventListenerResult = radioEventListener.toArray(
				new Port[radioEventListener.size()]);
		voiceEventListenerResult = voiceEventListener.toArray(
				new Port[voiceEventListener.size()]);
		civilianListenerResult = civilianVoiceListener.toArray(
				new CivilianVoiceListener[civilianVoiceListener.size()]);
		
		channelManager = new RandomChannelManager(me, world.getConfig());
	}

	AdvancedWorldModel getWorld() {
		return world;
	}

	StandardEntity getMe() {
		return me;
	}
	Port[] getRadioEventListener() {
		return radioEventListenerResult;
	}

	Port[] getVoiceEventListener() {
		return voiceEventListenerResult;
	}
	CivilianVoiceListener[] getCivilianVoiceListener() {
		return civilianListenerResult;
	}

	public CommunicationUtil getCommunicationUtil() {
		return comUtil;
	}

	public ChannelManager getChannelManager() {
		return channelManager;
	}
	
}
