package rione.geom;

import java.awt.Polygon;
import java.awt.Shape;

import rescuecore2.standard.entities.Blockade;

public class ExpandApexes {
	/**
	 * 閉塞のShapeを拡張する
	 * 
	 * @param blockade
	 *            Shapeを拡張する閉塞
	 * @param 拡張する大きさ
	 * @return 拡張されたShape
	 */
	public static Shape expandApexes(Blockade blockade, int size) {
		// 必要な情報が不足しているときは null を返す
		if(blockade == null || !blockade.isPositionDefined())return null;
		if(!blockade.isXDefined() || !blockade.isYDefined())return null;
		if(!blockade.isApexesDefined() || blockade.getApexes().length < 6 )return null;
		
		int cx = blockade.getX();
		int cy = blockade.getY();
		Polygon polygon = new Polygon();
		int[] apexes = blockade.getApexes();
		for (int i = 0; i < blockade.getApexes().length; i += 2) {
			int x = apexes[i];
			int y = apexes[i + 1];
			// Apexesから中心座標を引く
			int dx = x - cx;
			int dy = y - cy;
			// 中心座標とApexesの距離を取る
			double distance = getDistance(cx, cy, x, y);
			double coefficient = (distance + size) / distance;

			double newX = cx + (dx * coefficient);
			double newY = cy + (dy * coefficient);
			polygon.addPoint((int) newX, (int) newY);

			x = (apexes[i] + apexes[(i + 2) % apexes.length]) / 2;
			y = (apexes[i + 1] + apexes[(i + 3) % apexes.length]) / 2;
			dx = x - cx;
			dy = y - cy;
			distance = getDistance(cx, cy, x, y);
			coefficient = (distance + size) / distance;

			newX = cx + (dx * coefficient);
			newY = cy + (dy * coefficient);
			polygon.addPoint((int) newX, (int) newY);
		}
		return polygon;
	}
	private static double getDistance(int x1, int y1, int x2, int y2) {
		return Math.hypot((x1 - x2), (y1 - y2));
	}
}
