package rione.model;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import rescuecore2.misc.geometry.Line2D;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;

public class AdjacentGraph implements Map<Area, AdjacentLine[]> {

	/** 放射状の線は sqrt(建物の面積) * LINE_LENGTH_SCALE となる */
	//private DynamicConstant LINE_LENGTH_SCALE = DynamicConstant.create("1000.0 * 15.0");
	private static final double LINE_LENGTH_SCALE = 1000.0 * 2.0;
	private static final int rayCount = 32;
	
	final private Map<Area, AdjacentLine[]> flowGraph;
	
	public AdjacentGraph(AdvancedWorldModel world) {
		final Collection<StandardEntity> areas = world.getEntitiesOfType(StandardEntityURN.BUILDING, StandardEntityURN.ROAD);
		Map<Area, AdjacentLine[]> modifiableFlowGraph = new HashMap<Area, AdjacentLine[]>(areas.size());
		
		for (StandardEntity se : areas) {
			final Area target = (Area) se;
			//TODO isGroundAreaDefend
			final double r = (target instanceof Building)
					? Math.sqrt(((Building) target).getGroundArea()) * LINE_LENGTH_SCALE
					: world.getConfig().viewDistance;
			AdjacentLine[] lineArray = new AdjacentLine[rayCount];
			final double dAngle = Math.PI * 2 / rayCount;
			for (int i = 0; i < rayCount; ++i) {
				final double angle = i * dAngle;
				final double dx = Math.sin(angle) * r;
				final double dy = Math.cos(angle) * r;
				final Line2D ray = new Line2D(target.getX(), target.getY(), dx, dy);
				lineArray[i] = new AdjacentLine(target, world.getObjectsInRange(target, (int) r), ray);
			}
			modifiableFlowGraph.put(target, lineArray);
		}
		
		flowGraph = Collections.unmodifiableMap(modifiableFlowGraph);
	}
	
	@Override
	public void clear() {
		flowGraph.clear();
	}

	@Override
	public boolean containsKey(Object key) {
		return flowGraph.containsKey(key);
	}

	@Override
	public boolean containsValue(Object value) {
		return flowGraph.containsValue(value);
	}

	@Override
	public Set<Map.Entry<Area, AdjacentLine[]>> entrySet() {
		return flowGraph.entrySet();
	}

	@Override
	public AdjacentLine[] get(Object key) {
		return flowGraph.get(key);
	}

	@Override
	public boolean isEmpty() {
		return flowGraph.isEmpty();
	}

	@Override
	public Set<Area> keySet() {
		return flowGraph.keySet();
	}

	@Override
	public AdjacentLine[] put(Area key, AdjacentLine[] value) {
		return flowGraph.put(key, value);
	}

	@Override
	public void putAll(Map<? extends Area, ? extends AdjacentLine[]> m) {
		flowGraph.putAll(m);
	}

	@Override
	public AdjacentLine[] remove(Object key) {
		return flowGraph.remove(key);
	}

	@Override
	public int size() {
		return flowGraph.size();
	}

	@Override
	public Collection<AdjacentLine[]> values() {
		return flowGraph.values();
	}
	
}
