package rione.model;

import java.util.HashMap;
import java.util.Map;

import rescuecore2.config.Config;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;

/**
 * Confingを扱うクラス
 * @author utisam
 *
 */
public final class ConfigConstants {
	
	/**
	 * debugのフラグ<br>
	 * 定数なので，コンパイラがまともなら最適化でif文まるごと消してくれるはず
	 */
	public final boolean debug = false;
	
	/** シュミレーションのステップ数 */
	public final int timestep;
	/** エージェントの思考可能時間  */
	public final int thinkTime;
	
	/** チャンネルに関する接頭語 */
	private static final String CHANNELS_KEY_PREFIX = "comms.channels.";

	public final int ignoreUntil;
	/** 使用可能なチャンネル数 */
	public final int channelCount;
	/** Platoonがsubscribe（受信）可能なチャンネル数． */
	public final int subscribePlatoonSize;
	/** Centerがsubscribe（受信）可能なチャンネル数． */
	public final int subscribeCenterSize;
	/** 音声チャネル */
	public final Map<Integer, VoiceChannel> voiceChannels = new HashMap<Integer, VoiceChannel>();
	/** 通信チャネル */
	public final Map<Integer, RadioChannel> radioChannels = new HashMap<Integer, RadioChannel>();
	
	/** 木材の発火温度 */
	public final double woodenIgnition;
	/** 鉄の発火温度 */
	public final double steelIgnition;
	/** コンクリの発火温度 */
	public final double concreteIgnition;
	
	/** 視界範囲 */
	public final int viewDistance;
	/** HPの知覚単位 */
	public final int hpPrecision;
	/** HPの最大値 */
	public final int hpMax;
	/** ダメージ知覚単位 */
	public final int damagePrecision;
	
	/** タンクの最大容量 */
	public final int maxTankCapacity;
	/** 放水の最大出力 */
	public final int maxPower;
	/** タンクの単位ステップあたりの回復量 */
	public final int tankRefillRate;
	/** 消火可能範囲 */
	public final int extinguishableDistance;
	/** 消火栓でのタンクの単位ステップあたりの回復量*/
	public final int tankRefillHydrantRate;

	/** clear1回あたりの効果 */
	public final int repairDistance;
	/** EntityID型clear可能範囲 */
	public final int repairRate;
	/** */
	public final int repairRad;
	
	
	public final int MIN_X, MAX_X, MIN_Y, MAX_Y;
	
	static public abstract class Channel {
		public final int channel;
		protected Channel(int c) {
			channel = c;
		}
	}
	static public class VoiceChannel extends Channel {
		public final int range;
		public final int size;
		public final int maxNum;
		private VoiceChannel(int channel, final Config config) {
			super(channel);
			final String RANGE_SUFFIX = ".range";
			final String MESSAGE_SIZE_SUFFIX = ".messages.size";
			final String MESSAGE_MAX_SUFFIX = ".messages.max";
			range = config.getIntValue(CHANNELS_KEY_PREFIX + channel + RANGE_SUFFIX);
			size = config.getIntValue(CHANNELS_KEY_PREFIX + channel + MESSAGE_SIZE_SUFFIX);
			maxNum = config.getIntValue(CHANNELS_KEY_PREFIX + channel + MESSAGE_MAX_SUFFIX);
		}
	}
	static public class RadioChannel extends Channel {
		final public int bandwidth;
		private RadioChannel(int channel, final Config config) {
			super(channel);
			final String RADIO_BAND_WIDTH_KEY = ".bandwidth";
			bandwidth = config.getIntValue(CHANNELS_KEY_PREFIX + channel + RADIO_BAND_WIDTH_KEY);
		}
	}
	
	public ConfigConstants(final Config config, AdvancedWorldModel world) {
		// general
		final String IGNORE_UNTIL_KEY = "kernel.agents.ignoreuntil";
		final int DEFAULT_IGNORE_UNTIL = 3;
		ignoreUntil = config.getIntValue(IGNORE_UNTIL_KEY, DEFAULT_IGNORE_UNTIL);
		final String TIMESTEP_KEY = "kernel.timesteps";
		final int DEFAULT_TIMESTEP = 300;
		timestep = config.getIntValue(TIMESTEP_KEY, DEFAULT_TIMESTEP);
		/** エージェント思考可能時間のキー */
		final String THINK_TIME_KEY = "kernel.agents.think-time";
		/** 標準のエージェント思考可能時間 1000[ms] */
		final int DEFAULT_THINK_TIME = 1000;
		thinkTime = config.getIntValue(THINK_TIME_KEY, DEFAULT_THINK_TIME);
		
		// communication
		final String CHANNELS_COUNT_KEY = CHANNELS_KEY_PREFIX + "count";
		channelCount = config.getIntValue(CHANNELS_COUNT_KEY);
		final String SUBSCRIBE_PLATOON_KEY = CHANNELS_KEY_PREFIX + "max.platoon";
		subscribePlatoonSize = config.getIntValue(SUBSCRIBE_PLATOON_KEY, 0);
		final String SUBSCRIBE_CENTER_KEY = CHANNELS_KEY_PREFIX + "max.centre";
		subscribeCenterSize = config.getIntValue(SUBSCRIBE_CENTER_KEY, 0);
		final String CHANNELS_TYPE_KEY = ".type";
		for (int i = 0; i < channelCount; i++) {
			final String type = config.getValue(CHANNELS_KEY_PREFIX + i + CHANNELS_TYPE_KEY);
			if (type.startsWith("voice")) {
				voiceChannels.put(i, new VoiceChannel(i, config));
			} else if (type.startsWith("radio")) {
				radioChannels.put(i, new RadioChannel(i, config));
			}
		}
		
		// building
		final String WOODEN_IGNITION_KEY = "resq-fire.wooden_ignition";
		final double DEFAULT_WOODEN_IGNITION = 47.0f;
		woodenIgnition = config.getFloatValue(WOODEN_IGNITION_KEY, DEFAULT_WOODEN_IGNITION);
		final String STEEL_IGNITION_KEY = "resq-fire.steel_ignition";
		final double DEFAULT_STEEL_IGNITION = 47.0f;
		steelIgnition = config.getFloatValue(STEEL_IGNITION_KEY, DEFAULT_STEEL_IGNITION);
		final String CONCRETE_IGNITION_KEY = "resq-fire.concrete_ignition";
		final double DEFAULT_CONCRETE_IGNITION = 47.0f;
		concreteIgnition = config.getFloatValue(CONCRETE_IGNITION_KEY, DEFAULT_CONCRETE_IGNITION);
		
		// platoon
		final String VIEW_DISTANCE_KEY = "perception.los.max-distance";
	    final int DEFAULT_VIEW_DISTANCE = 30000;
		viewDistance = config.getIntValue(VIEW_DISTANCE_KEY, DEFAULT_VIEW_DISTANCE);
		final String HP_PERSEPTION_KEY = "perception.los.precision.hp";
		final int DEFAULT_HP_PERSEPTION = 1000;
		hpPrecision = config.getIntValue(HP_PERSEPTION_KEY, DEFAULT_HP_PERSEPTION); 
		final String DAMAGE_PERSEPTION_KEY = "perception.los.precision.damage";
		final int DEFAULT_DAMAGE_PERSEPTION = 100;
		damagePrecision = config.getIntValue(DAMAGE_PERSEPTION_KEY, DEFAULT_DAMAGE_PERSEPTION);
		
		// fire brigade
		final String MAX_WATER_KEY = "fire.tank.maximum";
		final int DEFAULT_TANK_CAPACITY = 15000;
		maxTankCapacity = config.getIntValue(MAX_WATER_KEY, DEFAULT_TANK_CAPACITY);
		final String MAX_EXTINGUISHABLE_KEY = "fire.extinguish.max-distance";
		final int DEFAULT_EXTINGUISHABLE_DISTANCE = 30000;
		extinguishableDistance = config.getIntValue(MAX_EXTINGUISHABLE_KEY, DEFAULT_EXTINGUISHABLE_DISTANCE);
		final String MAX_POWER_KEY = "fire.extinguish.max-sum";
		final int DEFAULT_POWER = 1000;
		maxPower = config.getIntValue(MAX_POWER_KEY, DEFAULT_POWER);
		final String WATER_REFILL_KEY = "fire.tank.refill-rate";
		final int DEFAULT_REFILL_RATE = 1000;
		tankRefillRate = config.getIntValue(WATER_REFILL_KEY, DEFAULT_REFILL_RATE);
		final String WATER_REFILL_HYDRANT_KEY = "fire.tank.hydrant-refill-rate";
		final int DEFAULT_REFILL_HYDRANT_RATE = 250;
		tankRefillHydrantRate = config.getIntValue(WATER_REFILL_HYDRANT_KEY, DEFAULT_REFILL_HYDRANT_RATE);		

		// police force
		final String DISTANCE_KEY = "clear.repair.distance";
		final int DEFAULT_REPAIR_DISTANCE = 10000;
		repairDistance = config.getIntValue(DISTANCE_KEY, DEFAULT_REPAIR_DISTANCE);
		final String REPAIR_RATE_KEY = "clear.repair.rate";
		final int DEFAULT_REPAIR_RATE = 10;
		repairRate = config.getIntValue(REPAIR_RATE_KEY, DEFAULT_REPAIR_RATE);
		final String DISTANCE_RAD_KEY = "clear.repair.rad";
		final int DEFAULT_REPAIR_RAD = 2000;
		repairRad = config.getIntValue(DISTANCE_RAD_KEY, DEFAULT_REPAIR_RAD);
		
		
		MIN_X = world.getWorldBounds().first().first();
		MIN_Y = world.getWorldBounds().first().second();
		MAX_X = world.getWorldBounds().second().first();
		MAX_Y = world.getWorldBounds().second().second();
		
		int maxHP = 10000;
		for (StandardEntity se: world.getEntitiesOfType(AgentConstants.HUMANOIDS)) {
			Human hm = (Human) se;
			if (hm.isHPDefined()) {
				maxHP = Math.max(maxHP, hm.getHP());
			}
		}
		hpMax = maxHP;
	}
	
}
