package rione.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;

final public class Entrance extends HashMap<Building, Set<Road>> {
	private static final long serialVersionUID = 1L;

	protected AdvancedWorldModel world = null;

	Entrance(AdvancedWorldModel w) {
		world = w;
		for (StandardEntity entity : world.getEntitiesOfType(AgentConstants.BUILDINGS)) {
			Building building = (Building) entity;
			this.put((Building) entity, setEntrance(building));
		}
	}

	/**
	 * 建物の入り口を設定する。<br>
	 * 建物と直接接続している場合を考慮し、 深さ優先で探索する。
	 * 
	 * @param building
	 *            入り口を設定する建物
	 * @return 建物の入り口の集合
	 */
	private Set<Road> setEntrance(Building building) {
		Set<Road> roads = new HashSet<Road>();
		Stack<Area> stack = new Stack<Area>();
		Set<Area> visited = new HashSet<Area>();
		stack.add(building);
		do {
			Area entity = stack.pop();
			visited.add(entity);
			if (entity instanceof Road) {
				roads.add((Road) entity);
			}
			else if (entity instanceof Building) {
				Building bld = (Building) entity;
				for (EntityID id : bld.getNeighbours()) {
					Area e = (Area) world.getEntity(id);
					if (!visited.contains(e)) {
						stack.push(e);
					}
				}
			}
		} while (!stack.isEmpty());
		return roads;
	}
	
	/**
	 * 建物の入り口のRoadを得る。<br>
	 * 複数の入り口がある場合を考慮し、Setで返す。
	 * 
	 * @param building
	 *            入り口を得る建物
	 * @return 入り口の集合
	 */
	public Set<Road> getEntrance(Building building) {
		return this.get(building);
	}

}
