package rione.model.route.area;

import java.util.*;

import rescuecore2.standard.entities.Area;
import rescuecore2.worldmodel.EntityID;

/**
 * Areaベースのルートを表すクラス
 * @author com
 *
 */
public class AreaRoute implements Comparable<AreaRoute> {

	/** 現在ルートの終点 */
	public final Area end;
	/** ひとつ前のルート */
	public final AreaRoute previous;
    public int distanceToGoal;
        
    /** 現在地点までの累積コスト */
    private int cost;
	
    /**
     * A* 始点コンストラクタ
     * @param area
     */
	public AreaRoute(Area origin, Area destination) {
        this.end = origin;
        this.previous = null;
        this.distanceToGoal = distance(origin.getX(), origin.getY(), destination.getX(), destination.getY());
	}
	
    /**
     * ダイクストラ始点コンストラクタ
     * @param area
     */
	public AreaRoute(Area end) {
        this.end = end;
        this.previous = null;
        this.distanceToGoal = 0;
	}
    
    /**
     * A*探索 拡張コンストラクタ
     * @param end
     * @param cost
     * @param previous
     */
    private AreaRoute(Area end, AreaRoute previous, int cost, Area destination) {
    	this.end = end;
    	this.previous = previous;
        this.cost = cost;
        this.distanceToGoal = distance(end.getX(), end.getY(), destination.getX(), destination.getY());
    }
    
    /**
     * ダイクストラ 拡張コンストラクタ
     * @param end
     * @param cost
     * @param previous
     */
    private AreaRoute(Area end, AreaRoute previous, int cost) {
    	this.end = end;
    	this.previous = previous;
        this.cost = cost;
        this.distanceToGoal = 0;
    }
    
    /**
     * ルートの拡張 A*
     * @param next
     * @param previous
     * @param costFunc
     * @param destination
     * @return
     */
    public AreaRoute expandedRoute(Area next, AreaRoute previous, CostFunction costFunc, Area destination) {
    	// 次の目標地点、コストの足し算、今までのルート
    	if (previous.previous != null) {
    		return new AreaRoute(next, previous, cost + costFunc.cost(previous.previous.end, previous.end, next), destination);
    	}
    	return new AreaRoute(next, previous, cost+costFunc.cost(previous.end, next), destination);
    }
    
    /**
     * ルートの拡張  ダイクストラ
     * @param next
     * @param previous
     * @param costFunc
     * @param destination
     * @return
     */
    public AreaRoute expandedRoute(Area next, AreaRoute previous, CostFunction costFunc) {
    	// 次の目標地点、コストの足し算、今までのルート
    	if (previous.previous != null) {
    		return new AreaRoute(next, previous, cost + costFunc.cost(previous.previous.end, previous.end, next));
    	}
    	return new AreaRoute(next, previous, cost+costFunc.cost(previous.end, next));
    }
    
    /**
     * 最終的なルートを得る
     * @return
     */
    public List<EntityID> getRoute() {
    	LinkedList<EntityID> route = new LinkedList<EntityID>();
    	for (AreaRoute rt=this; rt!=null; rt=rt.previous) {
    		route.addFirst(rt.end.getID());
    	}
    	//System.out.println("cost\t"+cost);
    	return route;
    }

    public int cost() {
    	return cost + distanceToGoal;
    }
    
    private int distance(int x1, int y1, int x2, int y2) {
        return (int)(Math.hypot(x1-x2, y1-y2));
    }
    
	@Override
	public boolean equals(Object obj) {
		if (obj == null) return false;
		AreaRoute route = (AreaRoute) obj;
		return end.equals(route.end);
	}

	@Override
	public int hashCode() {
		return end.getID().getValue();
	}

	@Override
	public int compareTo(AreaRoute route) {
		return (this.cost() - route.cost());
	}
	
}
