  package rione.model.route.area;

import java.util.*;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import rione.model.*;

/**
 * Areaベースのルートプラン生成をするクラス
 * @author com
 *
 */
public class AreaRouter {

	/**
	 * A*探索
	 * 単一の目的地に移動する際に用いる
	 * @param origin
	 * @param destinations
	 * @param costFunc
	 * @param world
	 * @return
	 */
	public static List<EntityID> get(Area origin, Area destination, CostFunction costFunc , AdvancedWorldModel world, int thinkTime) {
		long before = System.currentTimeMillis();
		PriorityQueue<AreaRoute> open = new PriorityQueue<AreaRoute>();
		Set<StandardEntity> close = new HashSet<StandardEntity>();
		
		// オープンリストに追加
		open.offer(new AreaRoute(origin, destination));
		do {
			AreaRoute route = open.poll(); // 要素を取り出して削除
			close.add(route.end);
			if (route.end.equals(destination)) { // 目標地点に到達
				return route.getRoute();
			}
			// 現在値からつながっている全てのAreaを調べる
			for (EntityID id : route.end.getNeighbours()) {
				Area next = (Area)world.getEntity(id);
				// 1つ前にたどったAreaならば無視
				if (route.previous != null && route.previous.end.getID().equals(id)) {
					continue;
				}
				// 次のArea（新しい探索先）をコストをつけてQueueに追加
				update(open, close, route.expandedRoute(next, route, costFunc, destination));
			}
			// 計算時間の制限に引っ掛かりそうなときは生成途中のRouteを返す
			// TODO 値決め打ちなのを直す
			if (System.currentTimeMillis() - before > thinkTime / 2) {
				System.out.println("A* Route plan create time over.");
				return route.getRoute();
			}
		} while (!open.isEmpty());
		return new ArrayList<EntityID>(Collections.singleton(origin.getID()));
	}
	
	/**
	 * ダイクストラ法
	 * 複数の目的地のどれかに移動する際に用いる
	 * @param origin
	 * @param destinations
	 * @param costFunc
	 * @param world
	 * @return
	 */
	public static List<EntityID> get(Area origin, Set<StandardEntity> destinations, CostFunction costFunc , AdvancedWorldModel world, int thinkTime) {
		long before = System.currentTimeMillis();
		PriorityQueue<AreaRoute> open = new PriorityQueue<AreaRoute>();
		Set<StandardEntity> close = new HashSet<StandardEntity>();
		
		// オープンリストに追加
		open.offer(new AreaRoute(origin));
		do {
			AreaRoute route = open.poll(); // 要素を取り出して削除
			close.add(route.end);
			if (destinations.contains(route.end)) { // 目標地点に到達
				return route.getRoute();
			}
			for (EntityID id : route.end.getNeighbours()) {
				Area next = (Area)world.getEntity(id);
				if (route.previous != null && route.previous.end.getID().equals(id)) {
					continue;
				}
				update(open, close, route.expandedRoute(next, route, costFunc));
			}
			if (System.currentTimeMillis() - before > thinkTime / 2) {
				System.out.println("Route plan create time over.");
				return route.getRoute();
			}
		} while (!open.isEmpty());
		return new ArrayList<EntityID>(Collections.singleton(origin.getID()));
	}
	
	private static void update(PriorityQueue<AreaRoute> open, Set<StandardEntity> close, AreaRoute route) {
		if (close.contains(route.end)) {
			return;
		}
		open.offer(route);
	}
}
