package rione.model.route.pov;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import rescuecore2.worldmodel.EntityID;
import rione.model.route.pov.graph.AreaNode;
import rione.model.route.pov.graph.PointNode;


public class POVPath implements Comparable<POVPath> {

	/** 現在地 */
	private final PointNode point;
	public PointNode getPoint() {
		return point;
	}

	/** このルートの道のり */
	private final double routeCost;
	/** 予想される残りのrouteCost(このルートから目標までの距離など) */
	private final double distance;
	/** 一つ前の経路ノード */
	private POVPath previous;


	/**
	 * 根用
	 * @param from
	 * 	開始位置
	 * @param destination
	 * 	最終目的地
	 */
	public POVPath(AreaNode from, double distance) {
		this.point = from;
        this.setPrevious(null);
        this.routeCost = 0.0;
		this.distance = distance;
	}

	/**
	 * 拡張用
	 * @param previous
	 * 	これまでの経路
	 * @param next
	 * 	追加する点
	 * @param costFunc
	 * 	コスト計算時の関数
	 * @param destination
	 * 	最終目的地
	 */
	public POVPath(POVPath previous, PointNode next, CostFunction costFunc, double distance) {
		this.setPrevious(previous);
		this.point = next;
		this.routeCost = previous.routeCost + costFunc.cost(previous.point, next);
		this.distance = distance;
	}
	
	void setPrevious(POVPath previous) {
		// TODO 自動生成されたメソッド・スタブ
		this.previous = previous;
	}

	public List<EntityID> getRoute() {
		LinkedList<EntityID> route = new LinkedList<EntityID>();
    	for (POVPath rt = this; rt != null; rt = rt.getPrevious()) {
    		if (rt.point instanceof AreaNode) {
    			route.addFirst(((AreaNode)rt.point).getBelong().getID());    			
    		}
    	}
    	return route;
	}
	
	public List<PointNode> getPoints() {
		LinkedList<PointNode> route = new LinkedList<PointNode>();
    	for (POVPath rt = this; rt != null; rt = rt.getPrevious()) {
   			route.addFirst(rt.point);    			
    	}
    	return route;
	}
	
	/**
	 * MultiAStar専用
	 * @return
	 */
	public List<EntityID> getReverseRoute() {
		List<EntityID> route = getRoute();
		Collections.reverse(route);
    	return route;
	}
	/**
	 * MultiAStar専用
	 * @return
	 */
	public List<PointNode> getReversePoints() {
		List<PointNode> points = getPoints();
		Collections.reverse(points);
    	return points;
	}
	
	public double cost() {
    	return routeCost + distance;
    }
	
	@Override
	public int compareTo(POVPath route) {
		return (int) (this.cost() - route.cost());
	}

	public POVPath getPrevious() {
		return previous;
	}

	public POVPath add(final POVPath newTail) {
		POVPath current = newTail;
		for (; current.getPrevious() != null; current = current.getPrevious())
			;
		current.setPrevious(this.previous);
		return newTail;
	}
}
