package rione.model.route.pov.reachable;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;

import rescuecore2.misc.Pair;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import rione.model.AdvancedWorldModel;
import rione.model.AgentConstants;
import rione.model.route.pov.graph.AreaNode;
import rione.model.route.pov.graph.EdgeNode;
import rione.model.route.pov.graph.PassableDictionary;
import rione.util.UnionFindTree;
import rione.viewer.component.extension.BorderExtension;
import rione.viewer.component.extension.ExtensionMap;

/**
 * 差分型のReachableArea<br>
 * 早いがPASSABLEからUNPASSABLEになると修正不可能になる．
 * 巨大マップ専用．
 * @author utisam
 *
 */
public class UFTReachableArea {
	private UnionFindTree<EntityID> sureReachableTree;
	
	public UFTReachableArea(AdvancedWorldModel world) {
		Collection<StandardEntity> areas = world.getEntitiesOfType(AgentConstants.AREAS);
		ArrayList<EntityID> ids = new ArrayList<EntityID>(areas.size());
		for (StandardEntity se : areas) {
			ids.add(se.getID());
		}
		sureReachableTree = new UnionFindTree<EntityID>(ids);
	}
	
	public void update(final AdvancedWorldModel world, final Set<EdgeNode> newPassables) {
		if (world.getTime() > world.getConfig().ignoreUntil) {
			updateSureReachable(world, newPassables);
		}
	}

	private void updateSureReachable(final AdvancedWorldModel world, Set<EdgeNode> newPassables) {
		//sureReachableTree.resetAll();
		final PassableDictionary passableDic = world.getRouter().getPassableDic();
		for (EdgeNode edge : newPassables) {
			AreaNode first = null;
			for (AreaNode area : edge.getNeighbours()) {
				if (passableDic.getPassableLevel(area, edge).isPassable()) {
					if (first == null) {
						first = area;
					}
					else {
						sureReachableTree.unite(first.getBelong().getID(), area.getBelong().getID());
					}
				}
			}
		}
	}
	
		
	public boolean isSureReachable(EntityID id, EntityID id2) {
		if(sureReachableTree.same(id, id2)){
			getUftCommand = true;			
		}
		return sureReachableTree.same(id, id2);
	}
	
	public boolean getUftCommand = false;
	private Pair<Integer, Integer> preLocation = null;
	/**
	 * aftershockが発生した場合、UFTを初期化する
	 * @param location
	 */
	public void uftMoveError(Pair<Integer, Integer> location){
		if (preLocation != null && this.getUftCommand == true) {
			double dist = Math.hypot(location.first() - preLocation.first(),
					location.second() - preLocation.second());
			if(dist < 100.0){
				//System.out.println("uftReset");
				sureReachableTree.resetAll();
			}else{
				getUftCommand = false;
			}
		}
		preLocation = location;
	}

	
	public void extension(ExtensionMap exMap, Area pos) {
		for (EntityID id : sureReachableTree) {
			if (isSureReachable(pos.getID(), id)) {
				exMap.put(id, new BorderExtension(exMap.get(id), Color.BLUE, 2.0));
			}
		}
	}
	
}
