package rione.test;

import junit.framework.TestCase;

import org.junit.Test;

import rione.io.BitArrayInputStream;
import rione.io.BitArrayOutputStream;
import rione.util.BitUtil;

/**
 * rione.communicationのテスト
 * @author utisam
 *
 */
public class CommunicationTest extends TestCase {

	@Test
	public void testNeedBitSize() {
		assertEquals("0を表すのに1bit", BitUtil.needBitSize(0), 1);
		assertEquals("1を表すのに1bit", BitUtil.needBitSize(1), 1);
		assertEquals("2を表すのに2bit", BitUtil.needBitSize(2), 2);
		assertEquals("3を表すのに2bit", BitUtil.needBitSize(3), 2);
		assertEquals("4を表すのに3bit", BitUtil.needBitSize(4), 3);
		assertEquals("5を表すのに3bit", BitUtil.needBitSize(5), 3);
		assertEquals("6を表すのに3bit", BitUtil.needBitSize(6), 3);
		assertEquals("7を表すのに3bit", BitUtil.needBitSize(7), 3);
		assertEquals("8を表すのに4bit", BitUtil.needBitSize(8), 4);
	}

	@Test
	public void testBitArrayStream() {
		BitArrayOutputStream out = new BitArrayOutputStream();
		out.writeBit(123, 10);
		out.writeBit(1234567890, 32);
		out.writeBit(123, 10);
		assertEquals(out.getBitLength(), 52);
		assertEquals(out.getDatasize(), (out.getBitLength() / 8));
		byte[] data = out.getData();//flush
		assertEquals(out.getDatasize(), (out.getBitLength() / 8) + 1);
		assertEquals(out.getDatasize(), data.length);
		BitArrayInputStream in = new BitArrayInputStream(data);
		assertEquals(in.readBit(10), 123 & 0x3FF);
		assertEquals(in.readBit(32), 1234567890 & 0xFFFFFFFF);
		assertEquals(in.readBit(10), 123 & 0x3FF);
		in.reset(data);
		assertEquals(in.readBit(10), 123 & 0x3FF);
		assertEquals(in.readBit(32), 1234567890 & 0xFFFFFFFF);
		assertEquals(in.readBit(10), 123 & 0x3FF);
	}
	
}
