package rione.test;


import static org.junit.Assert.*;

import java.util.HashMap;

import org.junit.Before;
import org.junit.Test;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.properties.IntProperty;
import rione.model.AgentConstants;

public class EnergyFlowTest {

	
	private MockWorldModel world = null;

	@Before
	public void setUp() {
		world = MockWorldModel.getMockWorldModel();
	}
	
	@Test
	public void test() {
		long t = System.currentTimeMillis();
		for (StandardEntity se : world.getAllEntities()) {
			if (se instanceof Building) {
				assertEquals("火災が発生していない状況では全て0",
						0.0, world.getEnergyFlow().getOut((Building) se), 0.0);
			}
		}
		// 全てに火をつける
		ChangeSet changes = new ChangeSet();
		final int testCaseTemp = 400;
		for (StandardEntity se : world.getEntitiesOfType(StandardEntityURN.BUILDING)) {
			Building building = (Building) se;
			IntProperty temp = building.getTemperatureProperty();
			temp.setValue(testCaseTemp);
			changes.addChange(building, temp);
			IntProperty fieryness = building.getFierynessProperty();
			fieryness.setValue(StandardEntityConstants.Fieryness.HEATING.ordinal());
			changes.addChange(building, fieryness);
		}
		world.merge(changes);
		world.getEnergyFlow().update(changes);
		HashMap<Building, Double> scores = new HashMap<Building, Double>();
		int notZeroCount = 0;
		for (StandardEntity se : world.getEntitiesOfType(AgentConstants.BUILDINGS)) {
			if (se.getClass().equals(Building.class)) {
				double affects = world.getEnergyFlow().getOut((Building) se);
				if (affects != 0.0) {
					notZeroCount++;
				}
				scores.put((Building) se, affects);
			}
			else {
				assertEquals(0.0, world.getEnergyFlow().getOut((Building) se), 0.0);
			}
		}
		assertTrue("全てのBuildingが火災状態の時，Buildingの80%以上の影響が0にならない．" +
				"ここで失敗するようなマップが存在するなら線分の長さを伸ばすべき．",
				80 < notZeroCount * 100 / world.getEntitiesOfType(AgentConstants.BUILDINGS).size());
		// 温度を2倍にする
		changes = new ChangeSet();
		for (StandardEntity se : world.getEntitiesOfType(StandardEntityURN.BUILDING)) {
			Building building = (Building) se;
			IntProperty temp = building.getTemperatureProperty();
			temp.setValue(testCaseTemp * 2);
			changes.addChange(building, temp);
			IntProperty fieryness = building.getFierynessProperty();
			fieryness.setValue(StandardEntityConstants.Fieryness.BURNING.ordinal());
			changes.addChange(building, fieryness);
		}
		world.merge(changes);
		world.getEnergyFlow().update(changes);
		for (StandardEntity se : world.getEntitiesOfType(AgentConstants.BUILDINGS)) {
			if (se.getClass().equals(Building.class)) {
				double affects = world.getEnergyFlow().getOut((Building) se);
				assertEquals("温度が2倍になると影響も2倍になる",
						scores.get(se) * 2.0, affects, Math.getExponent(affects));
			}
			else {
				assertEquals(world.getEnergyFlow().getOut((Building) se), 0.0, 0.0);
			}
		}
		long time = System.currentTimeMillis() - t;
		System.out.println("time: " + time);
		assertTrue("2コア4スレッドで200[ms]以下に押さえてください", time < 200);
	}

}
