package rione.test;

import gis2.GMLWorldModelCreator;

import java.io.File;

import javax.swing.JFileChooser;

import kernel.KernelException;
import rescuecore2.config.Config;
import rescuecore2.config.ConfigException;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rione.model.AdvancedWorldModel;
import rione.model.BuriedHumans;
import rione.model.BurningBuildings;
import rione.model.ConfigConstants;
import rione.model.CriticalArea;
import rione.model.LocateInBlockade;
import rione.model.TimeStamp;
import rione.model.Uniform;
import rione.model.areapartition.AreaPartition;
import rione.model.areapartition.AreaPartitionBuilder;
import rione.model.flow.EnergyFlow;
import rione.model.route.pov.POVRouter;

public class MockWorldModel extends AdvancedWorldModel {
	
	static private MockWorldModel instance;
	
	static {
		Config mockConfig = new Config();
		String rcrsdir = System.getenv("RCRS_HOME");
		final String sep = File.separator;
		if (rcrsdir == null) {
			JFileChooser dialog = new JFileChooser();
			dialog.setName("Rescue dir");
			dialog.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			dialog.showOpenDialog(null);
			rcrsdir = dialog.getSelectedFile().getPath();
		}
		String bootdir = (new File(rcrsdir, "rescue-1.0a-2011" + sep + "boot")).getPath();
		try {
			mockConfig.read(new File(bootdir, "config" + sep + "kernel-inline.cfg"));
		} catch (ConfigException e1) {
			//e1.printStackTrace();
			//.svnを読んでしまう
		}
		String path = (new File(bootdir, mockConfig.getValue("gis.map.dir"))).getPath();
		System.out.print("read map ... ");
		System.out.println(path);
		mockConfig.setValue("gis.map.dir", path);
		GMLWorldModelCreator gis = new GMLWorldModelCreator();
		try {
			StandardWorldModel world = StandardWorldModel.createStandardWorldModel(gis.buildWorldModel(mockConfig));
			instance = new MockWorldModel();
			instance.addEntities(world.getAllEntities());
			instance.initialize(null, mockConfig);
		}
		catch (KernelException e) {
			e.printStackTrace();
			instance = null;
		}
	}
	
	static MockWorldModel getMockWorldModel() {
		return instance;
	}
	
	@Override
	public void initialize(StandardEntity me, Config conf) {
		//TODO AdvancedWorldModelとなるべく一致させる
		assert me == null;// Mockの場合meにはnullが来ます
		config = new ConfigConstants(conf, this);
		
		uniform = new Uniform(this);
		timestamp = new TimeStamp(this);
		buriedHumans = new BuriedHumans(this);
		burningBuildings = new BurningBuildings(this);
		router = new POVRouter(null, this);
		criticalArea = new CriticalArea(this);
		locateInBlockade = new LocateInBlockade(this);
		areaPartitions = new AreaPartition(new AreaPartitionBuilder(this));
		energyFlow = new EnergyFlow(this);
	}
}
