package rione.test;

import java.awt.Point;
import java.util.HashMap;
import java.util.HashSet;

import junit.framework.TestCase;

import org.junit.Test;

import rione.model.route.pov.graph.PointNode;
import rione.util.SetPair;


/**
 * 使う前に確かめるためのテスト<br>
 * 将来同じ事を確かめるために残しておいてください．
 * @author utisam
 *
 */
public class UtilTest extends TestCase {

	
	/**
	 * Pointオブジェクトの生成するハッシュの挙動について確かめます．
	 */
	@Test
	public void testPointHash() {
		testPointHash(new Point(60800, 140800), new Point(60800, 140800));
		testPointHash(new Point(0, 0), new Point(0, 0));
		testPointHash(new Point(0xffffffff, 0), new Point(0xffffffff, 0));
		testPointHash(new Point(0x7fffffff, 0), new Point(0x7fffffff, 0));
	}
	private void testPointHash(Point a, Point b) {
		 assertEquals(a.hashCode(), a.hashCode());
		 assertTrue(b.equals(a));
		 assertEquals("ハッシュはObjectに(オブジェクトアドレス)よるものではなくPoint2Dのもの(座標による)である",
				 a.hashCode(), b.hashCode());
		 HashMap<Point, Integer> ptoi = new HashMap<Point, Integer>();
		 ptoi.put(a, 100);
		 assertEquals("Point2Dのハッシュは座標によるため，別オブジェクトでもよい．",
				 (int) ptoi.get(b), 100);
	}
	
	/**
	 * SetPairが正しく動作するかテストします
	 */
	@Test
	public void testSetPair() {
		SetPair<PointNode, PointNode> p1;
		SetPair<PointNode, PointNode> p2;
		
		// 一致テスト
		
		// 同じ値
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, 0, 0));
		testEqualsSetPair(p1, p2);
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 10, 10), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 10, 10), new PointNode(false, 0, 0));
		testEqualsSetPair(p1, p2);
		// 順序関係を入れ替え
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 100, 100), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, 100, 100));
		testEqualsSetPair(p1, p2);
		// 負の値
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, -100, -100), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, -100, -100));
		testEqualsSetPair(p1, p2);
		
		// 不一致テスト
		
		// 違う値
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 100, 0), new PointNode(false, 0, 1));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 100), new PointNode(false, 0, 1));
		testNotEqualsSetPair(p1, p2);
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 1, 1), new PointNode(false, 1, 1));
		testNotEqualsSetPair(p1, p2);
		// 負の値
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, -100, 0), new PointNode(false, 0, -1));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, -100), new PointNode(false, 0, -1));
		testNotEqualsSetPair(p1, p2);
		p1 = new SetPair<PointNode, PointNode>(
				new PointNode(false, 0, 0), new PointNode(false, 0, 0));
		p2 = new SetPair<PointNode, PointNode>(
				new PointNode(false, -1, -1), new PointNode(false, -1, -1));
		testNotEqualsSetPair(p1, p2);
	}
	private void testEqualsSetPair(SetPair<?, ?> p1, SetPair<?, ?> p2) {
		assertEquals(p1, p2);
		HashSet<SetPair<?, ?>> set = new HashSet<SetPair<?, ?>>();
		set.add(p1);
		assertTrue(set.contains(p1));
		assertTrue(set.contains(p2));
		set.clear();
		set.add(p2);
		assertTrue(set.contains(p2));
		assertTrue(set.contains(p1));
	}
	private void testNotEqualsSetPair(SetPair<?, ?> p1, SetPair<?, ?> p2) {
		assertFalse(p1.equals(p2));
		HashSet<SetPair<?, ?>> set = new HashSet<SetPair<?, ?>>();
		set.add(p1);
		assertTrue(set.contains(p1));
		assertFalse(set.contains(p2));
		set.clear();
		set.add(p2);
		assertFalse(set.contains(p1));
		assertTrue(set.contains(p2));
	}
	
}
