package rione.util;

/**
 * 順序関係を考慮しないPair.
 * 一致判断のときに順番は無視します．
 * また，ハッシュの値も値の順序に関わらず一致します．<br>
 * 主に無向Edgeに利用します．
 * @author utisam
 *
 */
public class SetPair<T1, T2> {

	private final int hash;
	private T1 first;
	private T2 second;
	
	public SetPair(T1 first, T2 second) {
		// Java6で例外が発生する演算は /, % の0割のみである
		hash = first.hashCode() ^ second.hashCode();//交換法則
		this.first = first;
		this.second = second;
	}
	@Override
	public int hashCode() {
		return hash;
	}
	@Override
	public boolean equals(Object o) {
		if (o instanceof SetPair<?, ?>) {
			@SuppressWarnings("unchecked")
			SetPair<T1, T2> e = (SetPair<T1, T2>) o;
			return (this.first().equals(e.first()) && this.second().equals(e.second()))
				|| (this.first().equals(e.second()) && this.second().equals(e.first()));
		}
		return false;
	}
	public T1 first() {
		return first;
	}
	private T2 second() {
		return second;
	}

}
